I2CEN 100,100 ' Enable I2C
I2CAddr  = &H20        ' this is the 23017 I2C address
                       ' A2=A1=A0=0 <-> x100 000x
Dim TBL(16)         ' an array to hold table data
Dim C_LineStart(4)  ' an array to hold the start addresses for each line
C_Lines = 0     ' variable to hold the number of lines on the LCD
C_LineLength = 0  ' variable to hold the line length of the LCD

Buttons = &h0
PortAStatus = 0
PortBStatus = 0

RSCMDmask = &B00000000 ' Select Command register
RSDATmask = &B10000000 ' Select Data register
Emask = &B00100000 ' Enable
BlueLed = &B00000001 ' Blue Backlight


' **** Change the parameters to indicate whether you have a 16 x 2
' **** or a 20 x 4 LCD screen.

InitLCDVariables(16, 2) ' <<<<<<<<<<-------

GoSub TBL
GoSub Init23017

InitLCD

PrintLCD 1, "  Hello World"
SetRGB 1, 1, 0 ' Set Backilight Colour (Green Red Blue)

Menu:

CheckButtons
If GetBit(Buttons, 0) = 0 Then PrintLCD 2, "Select"
If GetBit(Buttons, 4) = 0 Then PrintLCD 2, "Left"
If GetBit(Buttons, 3) = 0 Then PrintLCD 2, "Up"
If GetBit(Buttons, 2) = 0 Then PrintLCD 2, "Down"
If GetBit(Buttons, 1) = 0 Then PrintLCD 2, "Right"
If(Buttons And &B111111) = &B111111 Then PrintLCD 2, "      "


GoTo Menu

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Initialise the variables for this application
' (C_Prefix indicates constants).
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub InitLCDVariables(LCD_LineLength, LCD_Lines)

    C_LineLength = LCD_LineLength
    C_Lines = LCD_Lines

    C_LineStart(1) = 0        ' Start address for line 1
    C_LineStart(2) = 64       ' Start address for line 2
    C_LineStart(3) = 20       ' Start address for line 3
    C_LineStart(4) = 84       ' Start address for line 4

End Sub


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Initialise the LCD hardware
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub InitLCD
    LCD_Nibble &B0011, 0, 5           ' reset
    LCD_Nibble &B0011, 0, 5           ' reset
    LCD_Nibble &B0011, 0, 5           ' reset
    LCD_Nibble &B0010, 0, 2           ' 4 bit mode
    LCD_Nibble &B101000, 0
    LCD_Nibble &B1100, 0   ' display on, cursor on
    LCD_Nibble &B0110, 0
    LCD_Nibble &B0001, 0, 2
End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Move cursor to a location on the LCD screen.
' argument #1 is the line to be used (1 to 4)
' argument #2 is the column to be used (1 to 20)
' This Subroutine isn't used by the other code, but
' is included because it's simple and useful.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub LCDLocate(Line, Column)
  Local curPosn

  curPosn = 128 + C_LineStart(Line) + Column - 1
  LCD_Nibble Int(curPosn), 0

End Sub

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Display a line on the LCD
' argument #1 is the line to be used (1 to 4)
' argument #2 is the line to display (can be any length)
' Note: The text has to be padded out with spaces, otherwise
' 'odd' characters are displayed on the LCD.
' The code does the padding for you.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub PrintLCD ( LineNumber, Line$ )
  Local i, c, fillSpaces, fullLine$, curPosn

  curPosn = 128 + C_LineStart(LineNumber)


 ' first, send the cursor position (in two nibbles)
     LCD_Nibble Int(curPosn), 0

  fillSpaces = C_LineLength - Len(Line$) ' is the line full?
  If fillSpaces > 0 Then
    fullLine$ = Line$ + Space$(fillSpaces)
  Else
    fullLine$ = Line$
  EndIf

  ' then send the text character by character (two nibbles per character)
  For i = 1 To C_LineLength
    c = Asc(Mid$(fullLine$, i, 1))
  LCD_Nibble c, 1
  Next i

End Sub

'Addafruit LCD Plate
'RS=GPB7 EN=GPB5
'D4 - D7 = GPB4-GPB1
'RW=GPB6

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Send the lower 4 bits (called a nibble) to the LCD
' argument #1 is the nibble to send
' argument #2 is 1 if data, 0 if command (default is data)
' argument #3 is delay after the data has been sent (default is zero)
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Sub LCD_Nibble (Data, Flag, Wait_mSec)
  If Flag = 0 Then rsbit = RSCMDmask
  If Flag = 1 Then rsbit = RSDATmask

   ' put MSB OUT 1st
    temp = (Data \ &B10000)
    temp = TBL(temp) Or rsbit
    If Bon = 1 Then temp = temp Or &B00000001 ' Blue Led
GoSub DirectSendCmd

    ' put LSB
    temp = Data And &H0F
    temp = TBL(temp) Or rsbit
    If Bon = 1 Then temp = temp Or &B00000001 ' Blue LED

DirectSendCmd:
    temp = temp Xor Emask             ' E=1
 I2CSEND i2caddr, 0, 2, &H19, temp    ' send to 23017
    temp = temp Xor Emask             ' E=0
 I2CSEND i2caddr, 0, 2, &H19, temp
Return

If Wait_mSec > 0 Then Pause Wait_mSec ' modified 30/1/2012. Restore to Wait_mSec when Pause 0 bug corrected.
End Sub

Sub CheckButtons
I2CRCV i2caddr, 0, 1, Buttons, 1, &H09
End Sub


Sub SetRGB ( GLed, RLed, BLed )

I2CRCV i2caddr, 0, 1, PortAStatus, 1, &H19
I2CRCV i2caddr, 0, 1, PortBStatus, 1, &H09

If Gled = 0 Then PortAStatus = PortAStatus And &B01111111
If Gled = 1 Then PortAStatus = PortAStatus Or &B10000000

If Rled = 0 Then PortAStatus = PortAStatus And &B10111111
If Rled = 1 Then PortAStatus = PortAStatus Or &B01000000

If Bled = 0 Then PortBStatus = PortBStatus And &B11111110
If Bled = 1 Then PortBStatus = PortBStatus Or &B00000001
If Bled = 1 Then Bon = 1

I2CSEND i2caddr, 0, 2, &H09, PortAStatus    ' send to 23017
I2CSEND i2caddr, 0, 2, &H19, PortBStatus    ' send to 23017


End Sub

Function GetBit(byte_in, bit)
  ' Return the value (0/1) of a specified bit in a given byte
    GetBit = (byte_in And (2 ^ bit)) > 0
End Function


Init23017:

I2CSEND i2caddr, 0, 2, &H0A, &B10100000
I2CSEND i2caddr, 0, 2, &H0B, &B10100000

I2CSEND i2caddr, 0, 2, &H05, &B10100000
I2CSEND i2caddr, 0, 2, &H00, &B00111111
I2CSEND i2caddr, 0, 2, &H06, &B00111111

I2CSEND i2caddr, 0, 2, &H10, &B00000000
I2CSEND i2caddr, 0, 2, &H19, &B00000000

Return

TBL:
For I = 0 To 15
 Read TBL(I)
Next I
Return

Data &B00000000
Data &B00010000
Data &B00001000
Data &B00011000
Data &B00000100
Data &B00010100
Data &B00001100
Data &B00011100
Data &B00000010
Data &B00010010
Data &B00001010
Data &B00011010
Data &B00000110
Data &B00010110
Data &B00001110
Data &B00011110


